from discord import Embed, Color, Intents
from discord.ext import commands
from discord.utils import find
from dataclasses import dataclass
from yaml import load
from psutil import Process
from asyncio import sleep

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

intents = Intents.default()
intents.members = True

jarvis = commands.Bot(command_prefix=">", intents=intents)
jarvis_self = Process()


@dataclass
class Config:
    token: str
    client_id: str

    @classmethod
    def from_yaml(cls, y):
        instance = cls(token=y["token"], client_id=y["client_id"])
        return instance


def get_config(path: str = "config.yaml") -> Config:
    with open(path) as f:
        raw = f.read()
    y = load(raw, Loader=Loader)
    return Config.from_yaml(y)


def convert_bytesize(bytes: int) -> str:
    bytes = float(bytes)
    sizes = ["B", "KB", "MB", "GB"]
    size = 0
    while bytes >= 1024 and size < len(sizes) - 1:
        bytes = bytes / 1024
        size += 1
    return f"{bytes:0.2f}{sizes[size]}"


@jarvis.event
async def on_ready():
    print("Logged in as {0.user}".format(jarvis))
    print("Connected to {} guild(s)".format(len(jarvis.guilds)))


@jarvis.event
async def on_guild_join(guild):
    general = find(lambda x: x.name == "general", guild.channels)
    if general and general.permissions_for(guild.me).send_messages:
        await general.send(
            "Allow me to introduce myself. I am J.A.R.V.I.S., a virtual "
            + "artificial intelligence, and I'm here to assist you with a "
            + "variety of tasks as best I can, "
            + "24 hours a day, seven days a week."
        )
        await sleep(1)
        await general.send("Importing all preferences from home interface...")
        await sleep(5)
        await general.send("Systems are now fully operational")


@jarvis.command(name="status", help="Retrieve J.A.R.V.I.S. status")
async def _status(ctx):
    title = "J.A.R.V.I.S. Status"
    description = "All systems online"
    color = Color.from_rgb(152, 204, 218)
    embed = Embed(title=title, description=description, color=color)
    with jarvis_self.oneshot():
        embed.add_field(name="CPU Usage", value=jarvis_self.cpu_percent())
        embed.add_field(
            name="RAM Usage",
            value=convert_bytesize(jarvis_self.memory_info().rss),
        )
        embed.add_field(name="PID", value=jarvis_self.pid)
    await ctx.send(embed=embed)


if __name__ == "__main__":
    config = get_config()
    print(
        "https://discord.com/api/oauth2/authorize?client_id={}&permissions=8&scope=bot".format(
            config.client_id
        )
    )
    jarvis.run(config.token)
